/*************************************************************************
 *  TinyFugue - programmable mud client
 *  Copyright (C) 1993, 1994 Ken Keys
 *
 *  TinyFugue (aka "tf") is protected under the terms of the GNU
 *  General Public License.  See the file "COPYING" for details.
 ************************************************************************/
/* $Id: help.c,v 35004.1 1995/04/30 00:26:05 hawkeye Exp $ */

/*
 * Fugue help handling
 *
 * Uses the help index to search the helpfile for a topic.
 *
 * Original algorithm developed by Leo Plotkin.
 * Rewritten by Ken Keys to work with rest of program, and handle
 * topic aliasing, and subtopics.
 */

#include "config.h"
#include <ctype.h>
#include "port.h"
#include "dstring.h"
#include "variable.h"
#include "tfio.h"
#include "commands.h"

STATIC_BUFFER(indexfname);

static char *FDECL(readhelp, (char *s, FILE *stream));

static char *readhelp(s, stream)
    char *s;
    FILE *stream;
{
    if (!fgets(s, 81, stream)) return NULL;
    if (*s) s[strlen(s)-1] = '\0';
    return s;
}

int handle_help_command(args) 
    char *args;
{
    char *input, *major_buffer, *minor_buffer, *spare;
    char buf0[81], buf1[81], buf2[81];
    char *major_topic, *minor_topic, *place;
    CONST char *name;
    TFILE *helpfile, *indexfile;
    long offset = -1;

    Stringterm(indexfname, 0);
    buf0[0] = buf1[0] = buf2[0] = '\0';

    name = expand_filename(TFHELP);
    if ((helpfile = tfopen(name, "r")) == NULL) {
        operror(name);
        return 0;
    }
    Sprintf(indexfname, 0, "%s.idx", name);
    if ((indexfile = tfopen(indexfname->s, "r")) == NULL) {
        operror(indexfname->s);
        tfclose(helpfile);
        return 0;
    }

    name = (*args) ? args : "summary";

    input = buf0;
    major_buffer = buf1;
    minor_buffer = buf2;

    while (offset < 0 && readhelp(input, indexfile->u.fp) != NULL) {
        minor_buffer[0] = '\0';
        for (place = input; isdigit(*place); place++);
        if (*place == '@') {
            major_buffer[0] = '\0';
            spare = major_buffer;
            major_buffer = input;
        } else {
            spare = minor_buffer;
            minor_buffer = input;
        }
        ++place;
        if (ispunct(*place)) {
            if (strcmp(place, name) == 0 || strcmp(place + 1, name) == 0)
                offset = atol(input);
        } else {
            if (cstrcmp(place, name) == 0)
                offset = atol(input);
        }
        input = spare;
    }
    tfclose(indexfile);
    if (offset < 0) {
        oprintf("%% Help on subject %s not found.", name);
        tfclose(helpfile);
        return 0;
    }

    /* find offset, skip lines matching ^[@#], and remember last topic */
    tfjump(helpfile, offset);
    while (readhelp(input, helpfile->u.fp) != NULL) {
        if (*input != '@' && *input != '#') break;
        if (*minor_buffer) {
            spare = minor_buffer;
            minor_buffer = input;
            input = spare;
        } else if (*input == '@') {
            spare = major_buffer;
            major_buffer = input;
            input = spare;
        }
    }

    for (major_topic = major_buffer; isdigit(*major_topic); major_topic++);
    major_topic++;
    if (*minor_buffer) {
        for (minor_topic = minor_buffer; isdigit(*minor_topic); minor_topic++);
        minor_topic++;
        oprintf("Help on: %s: %s", major_topic, minor_topic);
    } else {
        oprintf("Help on: %s", major_topic);
    }

    do {
        if (*input == '@') break;
        else if (*input != '#') oputs(input);
        else if (*minor_buffer) break;
    } while (readhelp(input, helpfile->u.fp) != NULL);

    tfclose(helpfile);
    return 1;
}

#ifdef DMALLOC
void free_help()
{
    Stringfree(indexfname);
}
#endif
